/*
 * Copyright (c) 2004 Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: como_collector.c,v 1.6 2004/11/30 16:25:19 lrizzo Exp $
 */

#include <stdio.h>
#include <stdlib.h>     /* atoi */
#include <unistd.h>     /* getopt */
#include <string.h>	/* strcpy */

#include "qutils.h"
#include "query.h"


/*
 * -- parse_cmdline
 */
int parse_cmdline(int argc, char *argv[], char *fmt, int *port)
{
    int opt;

    /* default options */
    strcpy(fmt, "%d\n");
    *port = SENDTO_PORT;

    while ((opt = getopt(argc, argv, "f:p:h")) != -1) {
        switch (opt) {
            case 'f':
                snprintf(fmt, MAX_FMT, "%s", optarg);
                break;
            case 'p':
                *port = atoi(optarg);
                break;
            case 'h':
                fprintf(stderr, "\nCoMo collector for sendto queries\n\n");
            case '?':
            default:
                fprintf(stderr, "Usage: %s [-f format] [-p port]\n", argv[0]);
                fprintf(stderr, "\t-f: input/output format as in printf\n");
                fprintf(stderr, "\t-p: port number to listen for connections\n");
                fprintf(stderr, "\t-h: display this help and exit\n\n");
                fflush(stderr);
                return -1;
        }
    }

    return 0;
}


/*
 * -- main
 */
int main(int argc, char *argv[])
{
    char fmt[MAX_FMT];
    int port;
    int sd;
    qmsg_t m;

    /* check and get command line arguments */
    if (parse_cmdline(argc, argv, fmt, &port) < 0)
        exit(0);

    /* init server and wait for a connection from query process */
    if ((sd = init_server(port)) < 0) {
        perror("init_server");
        exit(0);
    }

    while (1) {
        /* first read the record header */
        if (read(sd, &m, QHDR_SIZE) != QHDR_SIZE) {
            perror("read_header");
            exit(0);
        }

        /* then read the record data */
        if (read(sd, m.msg.data, m.ctl.reclen) != (ssize_t)m.ctl.reclen) {
            perror("read_record");
            exit(0);
        }

        /* print record according to the given format */
        qfprintf(stdout, fmt, m.msg.data);
	fflush(stdout);
    }

    close(sd);
    return 0;
}
