/***************************************************************************
                          rrd_cmd.c  -  description
                             -------------------
    begin                : Mon Feb 2 2004
    copyright            : (C) 2004 by Technical Univ. of Catalonia (UPC)
    email                : pbarlet@ac.upc.es
 ***************************************************************************/

/***************************************************************************
                    This file is part of SMART software

  SMART is property of CCABA-UPC (http://www.ccaba.upc.es) for its usage,
  modifications, suggestions, etc. please contact first: pbarlet@ac.upc.es

 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>     /* bzero */
#include <unistd.h>     /* gethostname */
#include <netdb.h>      /* gethostbyname */
#include <sys/types.h>  /* socket, bind, listen, accept, stat */
#include <sys/socket.h> /* socket, bind, listen, accept */
#include <sys/time.h>   /* gettimeofday */
#include <sys/stat.h>   /* stat */
#include <netinet/in.h> /* struct sockaddr_in */
#include <math.h>       /* ceil */
#include <rrd.h>

#include "qutils.h"
#include "module.h"
#include "query.h"

#define MAX_APP     11
#define MAX_RRD_STR 256
#define MAX_RRD_ARG 1024
#define MAX_RRD_UPD 65535
#define MAX_RRD_TMP 65537
#define RRD_WIDTH   550
#define RRD_HEIGHT  200
#define RRD_PAD     20

extern int optind;
extern int opterr;

int color[] =
{
    0x118F93, 0xFF0000, 0xFFFF00, 0xAA0000, 0x0000FF,
    0xFF00FF, 0x00FFFF, 0x9B65FF, 0xFFDCA8, 0x98ABFF,
    0x888888, 0xC0C000, 0x00FF00, 0xEA7500, 0xAC2FBC,
    0x267F45, 0x26387F, 0xC00000, 0xFF0000, 0xFFC0C0,
    0x004000, 0x008000, 0x00C000, 0x00CCCC, 0x00FF00,
    0x0AFFFF, 0xC0FFC0, 0x000080, 0x0000C0, 0x0000FF,
    0xC0C0FF, 0x404000, 0x808000, 0xDCDCDC, 0xFFFF00,
    0xFFFF0F, 0xFFFFC0, 0x004040, 0x008080, 0x00C0C0,
    0x00FFFF, 0xC0FFFF, 0x800080, 0xFF00FF, 0xC000C0,
    0xFFC0FF, 0xC05800, 0xFF8000, 0xFFA858, 0xFFFDB0,
    0xC0FFC0, 0x00D200, 0xFFDCA8, 0x000000
};

char *app[] =
{
    "HTTP",
    "SSH",
    "Telnet",
    "Mail",
    "News",
    "Network",
    "File_Transfer",
    "File_Systems",
    "Tunneling",
    "P2P",
    "Unknown"
};


/*
 * -- rrd_show_error
 */
void
rrd_show_error(void)
{
    if (rrd_test_error())
    {
        fprintf(stderr, "RRD-ERROR: %s\n",rrd_get_error());
        fflush(stderr);
        rrd_clear_error();
    }
}


/*
 * -- rrd_init_db
 */
int
rrd_init_db(uint32_t ts, char *file)
{
    struct stat sb;
    char rrdarg[MAX_RRD_ARG][MAX_RRD_STR], *argv[MAX_RRD_ARG];
    uint step, hbeat, steps, rows, interval = 1;
    int argc, i;
    float xff;

    /* Init vars */
    step = interval;
    hbeat = ceil(step * 1.5);
    xff = 0.5;
    argc = 6;
    for (i = 0; i < MAX_RRD_ARG; i++)
        argv[i] = rrdarg[i];

    /* RRD create main arguments */
    strcpy  (rrdarg[0], "create");
    strcpy  (rrdarg[2], "-b");
    sprintf (rrdarg[3], "%u", ts);
    strcpy  (rrdarg[4], "-s");
    sprintf (rrdarg[5], "%d", interval);

    /* RRD create Data Sources */
    for (i = 0; i < MAX_APP; i++)
    {
        sprintf(rrdarg[argc++], "DS:%s_b:ABSOLUTE:%d:U:U", app[i], hbeat);
        sprintf(rrdarg[argc++], "DS:%s_p:ABSOLUTE:%d:U:U", app[i], hbeat);
    }

    /* Hourly RRA */
    steps = 1; //Mean: INTERVAL
    rows = ceil((double)3600/step);
    sprintf(rrdarg[argc++], "RRA:AVERAGE:%f:%d:%d", xff, steps, rows);

    /* Daily RRA */
    steps = ceil((double)300/step); //Mean: 5 min
    rows = 288; //12*24 = 288
    sprintf(rrdarg[argc++], "RRA:AVERAGE:%f:%d:%d", xff, steps, rows);

    /* Weekly RRA */
    steps = ceil((double)900/step); //Mean: 15 min
    rows = 672; //4*24*7 = 336
    sprintf(rrdarg[argc++], "RRA:AVERAGE:%f:%d:%d", xff, steps, rows);

    /* Monthly RRA */
    steps = ceil((double)3600/step); //Mean: 1 hour
    rows = 744; //24*31 = 744
    sprintf(rrdarg[argc++], "RRA:AVERAGE:%f:%d:%d", xff, steps, rows);

    /* Yearly RRA */
    steps = ceil((double)43200/step); //Mean: 12 hours
    rows = 732; //2*366 = 732
    sprintf(rrdarg[argc++], "RRA:AVERAGE:%f:%d:%d", xff, steps, rows);

    /* Check for previous file existence */
    if (stat(file, &sb))
    {
        optind = 0;
        opterr = 0;
        strcpy(rrdarg[1], file);
        if (rrd_create(argc, argv))
            return -1;
    }

    return 0;
}


/*
 * -- rrd_update_cmd
 */
int
rrd_update_cmd(qmsg_t *m, uint nrec, uint32_t ts, char *file)
{
    char rrdarg[5][MAX_RRD_UPD], *argv[MAX_RRD_ARG];
    char rrdtmp[MAX_RRD_TMP];
    char *data;
    uint i;

    /* Init vars */
    *rrdarg[3] = '\0';
    *rrdarg[4] = '\0';
    for (i = 0; i< MAX_RRD_ARG; i++)
        argv[i] = rrdarg[i];

    /* RRD update main arguments */
    strcpy  (rrdarg[0], "update");
    sprintf (rrdarg[1], "%s", file);
    strcpy  (rrdarg[2], "-t");
    sprintf (rrdarg[4], "%u", ts);

    /* RRD update */
    for (i = 0; i < nrec; i++)
    {
        data = m[i].msg.data;

        /* Template */
        sprintf(rrdtmp, "%s_b:%s_p:", app[*(uint8_t *)(data + 4)],
            app[*(uint8_t *)(data + 4)]);
        strcat(rrdarg[3], rrdtmp);

        /* Values */
        sprintf(rrdtmp, ":%llu:%llu", NTOHLL(*(uint64_t *)(data + 5)),
            NTOHLL(*(uint64_t *)(data + 13)));
        strcat(rrdarg[4], rrdtmp);
    }

    /* RRD update call */
    rrdarg[3][strlen(rrdarg[3])-1]= '\0'; //Delete last ":"
    optind = 0;
    opterr = 0;

    if (rrd_update(5, argv))
       return -1;

    return 0;
}


/*
 * -- rrd_graph_call
 */
int
rrd_graph_call(int argc, char **argv)
{
    int xsize, ysize, i;
    char **calcpr;

    optind = 0;
    opterr = 0;
    if (rrd_graph(argc, argv, &calcpr, &xsize, &ysize))
        return -1;

    if (calcpr != NULL)
    {
        for (i = 0; calcpr[i]; i++)
           free(calcpr[i]);
        free(calcpr);
    }

    return 0;
}


/*
 * -- rrd_graph_cmd
 */
int
rrd_graph_cmd(uint32_t ts, char *file, char *output, char *ival)
{
    struct timeval tv;
    struct tm *gtime;
    char date[MAX_RRD_STR], pad[MAX_RRD_STR];
    char rrdarg_b[MAX_RRD_ARG][MAX_RRD_STR];
    char rrdarg_p[MAX_RRD_ARG][MAX_RRD_STR];
    char *argv_b[MAX_RRD_ARG];
    char *argv_p[MAX_RRD_ARG];
    int argc_b, argc_p;
    int err, i;
    char *type;

    /* get current time as graph generation time */
    if ((err = gettimeofday(&tv, NULL)))
        return err;
    gtime = localtime(&tv.tv_sec);
    if (gtime == NULL)
        return -1;
    sprintf(date, "%04d-%02d-%02d %02d:%02d:%02d",
        gtime->tm_year+1900,
        gtime->tm_mon+1,
        gtime->tm_mday,
        gtime->tm_hour,
        gtime->tm_min,
        gtime->tm_sec
    );

    /* init vars */
    type = "AREA";
    argc_b = 18;
    argc_p = 18;
    for (i = 0; i< MAX_RRD_ARG; i++)
    {
        argv_b[i] = rrdarg_b[i];
        argv_p[i] = rrdarg_p[i];
    }
    for (i = 0; i < MAX_RRD_STR; i++)
        pad[i] = ' ';
    pad[MAX_RRD_STR-1] = '\0';

    /* RRD graph main arguments */
    strcpy  (rrdarg_b[0],  "graph");
    sprintf (rrdarg_b[1],  "%s_bps.png", output);
    strcpy  (rrdarg_b[2],  "-w");
    sprintf (rrdarg_b[3],  "%d", RRD_WIDTH);
    strcpy  (rrdarg_b[4],  "-h");
    sprintf (rrdarg_b[5],  "%d", RRD_HEIGHT);
    strcpy  (rrdarg_b[6],  "-s");
    sprintf (rrdarg_b[7],  "%u-%s", ts, ival);
    strcpy  (rrdarg_b[8],  "-e");
    sprintf (rrdarg_b[9],  "%u", ts);
    strcpy  (rrdarg_b[10], "-l");
    strcpy  (rrdarg_b[11], "0");
    strcpy  (rrdarg_b[12], "-t");
    sprintf (rrdarg_b[13], "Application breakdown (%s)", "bits/s");
    strcpy  (rrdarg_b[14], "-v");
    sprintf (rrdarg_b[15], "bits/sec");
    strcpy  (rrdarg_b[16], "-a");
    strcpy  (rrdarg_b[17], "PNG");
    for (i = 0; i < argc_b; i++)
        strcpy(rrdarg_p[i], rrdarg_b[i]);
    sprintf (rrdarg_p[1],  "%s_pps.png", output);
    sprintf (rrdarg_p[13], "Application breakdown (%s)", "pkts/s");
    sprintf (rrdarg_p[15], "pkts/s");

    /* print legend header */
    sprintf(rrdarg_b[argc_b++], "COMMENT:                ");
    sprintf(rrdarg_b[argc_b++], "COMMENT:          AVERAGE");
    sprintf(rrdarg_b[argc_b++], "COMMENT:          MAX");
    sprintf(rrdarg_b[argc_b++], "COMMENT:            LAST\\c");
    sprintf(rrdarg_p[argc_p++], "COMMENT:                ");
    sprintf(rrdarg_p[argc_p++], "COMMENT:          AVERAGE");
    sprintf(rrdarg_p[argc_p++], "COMMENT:          MAX");
    sprintf(rrdarg_p[argc_p++], "COMMENT:            LAST\\c");

    /* RRD Graph */
    for (i = 0; i < MAX_APP; i++)
    {
        /* bps traffic statistics */
        sprintf(rrdarg_b[argc_b++], "DEF:t%s=%s:%s_b:AVERAGE",
            app[i], file, app[i]);
        sprintf(rrdarg_b[argc_b++], "CDEF:b%s=t%s,UN,0,t%s,IF,8,*",
            app[i], app[i], app[i]);

        /* pps traffic statistics */
        sprintf(rrdarg_p[argc_p++], "DEF:t%s=%s:%s_p:AVERAGE",
            app[i], file, app[i]);
        sprintf(rrdarg_p[argc_p++], "CDEF:p%s=t%s,UN,0,t%s,IF",
            app[i], app[i], app[i]);

        /* bps plot style */
        sprintf(rrdarg_b[argc_b++], "%s:b%s#%06x:%s",
            type, app[i], color[i], app[i]);
        pad[RRD_PAD-strlen(app[i])] = '\0';
        sprintf(rrdarg_b[argc_b++], "COMMENT:%s", pad);
        pad[RRD_PAD-strlen(app[i])] = ' ';
        sprintf(rrdarg_b[argc_b++], "GPRINT:b%s:AVERAGE:%%8.2lf%%s", app[i]);
        sprintf(rrdarg_b[argc_b++], "COMMENT:    ");
        sprintf(rrdarg_b[argc_b++], "GPRINT:b%s:MAX:%%8.2lf%%s", app[i]);
        sprintf(rrdarg_b[argc_b++], "COMMENT:    ");
        sprintf(rrdarg_b[argc_b++], "GPRINT:b%s:LAST:%%8.2lf%%s\\c", app[i]);
        sprintf(rrdarg_b[argc_b++], "%s:b%s#%06x", type, app[i], color[i]);

        /* pps plot style */
        sprintf(rrdarg_p[argc_p++], "%s:p%s#%06x:%s",
            type, app[i], color[i], app[i]);
        pad[RRD_PAD-strlen(app[i])] = '\0';
        sprintf(rrdarg_p[argc_p++], "COMMENT:%s", pad);
        pad[RRD_PAD-strlen(app[i])] = ' ';
        sprintf(rrdarg_p[argc_p++], "GPRINT:p%s:AVERAGE:%%8.2lf%%s", app[i]);
        sprintf(rrdarg_p[argc_p++], "COMMENT:    ");
        sprintf(rrdarg_p[argc_p++], "GPRINT:p%s:MAX:%%8.2lf%%s", app[i]);
        sprintf(rrdarg_p[argc_p++], "COMMENT:    ");
        sprintf(rrdarg_p[argc_p++], "GPRINT:p%s:LAST:%%8.2lf%%s\\c", app[i]);
        sprintf(rrdarg_p[argc_p++], "%s:p%s#%06x", type, app[i], color[i]);

        /* Stacked graph */
        type = "STACK";
    }

    /* graph generation time */
    strcpy  (rrdarg_b[argc_b++], "COMMENT:\\s");
    sprintf (rrdarg_b[argc_b++], "COMMENT:generated %s by CoMo\\c", date);
    strcpy  (rrdarg_p[argc_p++], "COMMENT:\\s");
    sprintf (rrdarg_p[argc_p++], "COMMENT:generated %s by CoMo\\c", date);

    /* generate RRD graphs */
    rrd_graph_call(argc_b, argv_b);
    rrd_graph_call(argc_p, argv_p);

    return 0;
}


/*
 * -- main
 */
int
main(__unused int argc, __unused char *argv[])
{
    uint32_t    last_ts;
    qmsg_t      m[MAX_APP];
    int         sd;
    uint        i = 1;

    /* wait for a connection from query */
    if ((sd = init_server(SENDTO_PORT)) < 0)
    {
        perror("init_server");
        exit(0);
    }

    /* first read the record header */
    if (read(sd, &m[0], QHDR_SIZE) != QHDR_SIZE)
    {
        perror("read_header");
        exit(0);
    }

    /* read first record to get first timestamp */
    if (read(sd, m[0].msg.data, m[0].ctl.reclen) != (ssize_t)m[0].ctl.reclen)
    {
        perror("read_record");
        exit(0);
    }

    /* init ts with 1st pkt */
    last_ts = ntohl(*(uint32_t *)m[0].msg.data);

    /* init RRD database using the ts of the 1st pkt */
    if (rrd_init_db(last_ts - 1, "data/application.rrd") < 0)
    {
        rrd_show_error();
        exit(0);
    }

    /* read records from query and process it */
    while (1)
    {
        /* i = 0 contains 1st record of this ts */
        i = 1;

        /* read until timestamp changes */
        while (last_ts == ntohl(*(uint32_t *)m[i-1].msg.data))
        {
            /* first read the record header */
            if (read(sd, &m[i], QHDR_SIZE) != QHDR_SIZE)
            {
                perror("read_header");
                exit(0);
            }

            /* then read the record data */
            if (read(sd, m[i].msg.data, m[i].ctl.reclen) < (ssize_t)m[i].ctl.reclen)
            {
                perror("read_record");
                exit(0);
            }
            i++;
        }

        /* update RRD database with all data of current ts */
        if (rrd_update_cmd(m, i-1, last_ts, "data/application.rrd") < 0)
            rrd_show_error(); //error on update is tolerated

        /* PROVISIONAL: generate output graph */
        if (rrd_graph_cmd(last_ts, "data/application.rrd", "data/app", "1h") < 0)
            rrd_show_error(); //error on graph is tolerated

        /* move data of next ts to the front */
        m[0] = m[i-1];
        last_ts = ntohl(*(uint32_t *)m[0].msg.data);
    }

    //This code never will be reached...

    /* close opened sockets */
    close(sd);

    return 0;
}
