/*
 * Copyright (c) 2004 Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sniffers.c,v 1.15 2005/01/16 22:33:37 gianluca Exp $
 */

#include <string.h>     /* bcopy, bcmp */

#ifdef USEDAG
#include <dagapi.h>
#endif 

#include "sniffers.h"

#ifndef ETH_ALEN
#   define ETH_ALEN 6
#endif

/*
 * -- get_direction
 *
 * determine traffic direction on ethernet
 * XXX what is the logic here ???
 */
int
get_direction(struct _como_machdr *machdr)
{
    static char macaddr[ETH_ALEN];
    static int init = 0;

    if (!init) {
        bcopy(machdr->dst, macaddr, ETH_ALEN);
        init = 1;
        return 0;
    }

    return (bcmp(machdr->dst, macaddr, ETH_ALEN));
}


/*
 * -- skip_layertwo
 *
 * returns the length of the layer two header.
 * sniffers skip the layer two header to return
 * just the IP packet.
 *
 */
int
skip_layertwo(int type)
{
    switch (type) {
        case BPF_ETH:
            return 14;

#ifdef USEDAG	// XXX sure we need this ?
        case TYPE_LEGACY:
        case TYPE_HDLC_POS:
        case TYPE_ATM:
        case TYPE_AAL5:
        case TYPE_MC_HDLC:
        case TYPE_MC_RAW:
            return 0;

        case TYPE_ETH:
            return 16;
#endif

        default:
            return 0;
    }

    return 0;
}

extern sniffer_t bpf_sniffer;
extern sniffer_t pcap_sniffer;
extern sniffer_t dag_sniffer;
extern sniffer_t erf_sniffer;
extern sniffer_t libpcap_sniffer;
sniffer_t *__sniffers[] =
{
    &bpf_sniffer,
    &pcap_sniffer,
    &dag_sniffer,
    &erf_sniffer,
    &libpcap_sniffer,
    NULL
};
