/*
 * Copyright (c) 2004 Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: module.h,v 1.3 2005/01/24 11:58:04 dvercell Exp $
 */

#ifndef _COMO_MODULE_H
#define _COMO_MODULE_H

#include <stdio.h>	/* XXX for asprintf */
#include <string.h>		/* memcpy */

/*
 * Some useful macros to write modules
 */

/*
 * FLOWDESC/EFLOWDESC are supposed to be defined by individual
 * modules as the type to be used for flow descriptors. If so,
 * here we define an F()/EF() macro to cast a pointer to
 * (FLOWDESC/EFLOWDESC *) to make the writing of modules more
 * convenient.
 */
#define F(x)     ((FLOWDESC *)(x))
#define EF(x)    ((EFLOWDESC *)(x))

/*
 * One more indirection is needed within compare_fn()
 */
#define CMPEF(x) (*(EFLOWDESC **)(x))

/*
 * Macros to convert a uint64_t from host to network byte order
 * and vice-versa
 */
#define HTONLL(x)   ((uint64_t)htonl((uint32_t)((x) >> 32)) |   \
            (uint64_t)htonl((uint32_t)((x) & 0xffffffff)) << 32)

#define NTOHLL(x)   ((uint64_t)ntohl((uint32_t)(x >> 32)) |     \
            (uint64_t)ntohl((uint32_t)(x & 0xffffffff)) << 32)

/*
 * Macros to copy integers from host to network byte order.
 * This macros are supposed to be used by store()/estore()
 */
#define PUTH8(x, val) {         \
    uint8_t v = val;            \
    memcpy(x, &v, sizeof(v));   \
}

#define PUTH16(x, val) {        \
    uint16_t v = htons(val);    \
    memcpy(x, &v, sizeof(v));   \
}

#define PUTH32(x, val) {        \
    uint32_t v = htonl(val);    \
    memcpy(x, &v, sizeof(v));   \
}

#define PUTH64(x, val) {        \
    uint64_t v = HTONLL(val);   \
    memcpy(x, &v, sizeof(v));   \
}

/*
 * Macros to copy integers directly in network byte order
 * This macros are supposed to be used by store()/estore()
 */
#define PUTN8(x, val)   PUTH8(x, val)

#define PUTN16(x, val) {        \
    uint16_t v = val;           \
    memcpy(x, &v, sizeof(v));   \
}

#define PUTN32(x, val) {        \
    uint32_t v = val;           \
    memcpy(x, &v, sizeof(v));   \
}

#define PUTN64(x, val) {        \
    uint64_t v = val;           \
    memcpy(x, &v, sizeof(v));   \
}


#endif /* _COMO_MODULE_H */

