/*
 * Copyright (c) 2004 Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: query.h,v 1.6 2005/01/21 17:20:43 lrizzo Exp $
 */

#ifndef _COMO_QUERY_H
#define _COMO_QUERY_H

#ifdef linux
#include <stdint.h>		/* uint8_t, etc. */
#endif


/* 
 * well known port numbers
 */
#define QSERV_PORT  	44444
#define SENDTO_PORT 	55555

/* 
 * some constants... XXX they shouldn't be here. move them somewhere else.
 */
#define MODULENAME_MAX	1024
#define FILTERSTR_MAX	1024


/* 
 * message types 
 */
enum q_type_t {
	Q_BOGUS = 0,	/* something invalid */
	Q_QUERY = 1,	/* post query on export data */
	Q_ERR = 2,		/* error */
	Q_ACK = 3,		/* ack query */
	Q_DATA = 4,		/* record data */
	Q_ASCII = 5,	/* new-style query or reply */
	Q_MAX = 10		/* max id (below the ascii set) */
};


/* 
 * query request message 
 */
struct _query_msg
{
    enum q_type_t type:8; 		/* message type */
    uint16_t len; 		/* message length */
    char module[MODULENAME_MAX];/* module name */
    char filter[FILTERSTR_MAX]; /* filter expression */
    uint32_t start;             /* query starts at */
    uint32_t end;               /* query ends at */
    char args[0]; 		/* query arguments */
};


/* 
 * data record message 
 */
struct _query_dat 
{
    enum q_type_t type; 		/* message type */
    uint16_t len; 		/* message length */
    char rec[0];		/* record */
};


/* 
 * control message (ACKs, ERRs)
 */
struct _query_ctl
{
    uint8_t type; 		/* message type */
    uint8_t pad; 		/* padding */
    uint16_t code; 		/* error code */
};


typedef struct _query_msg qreq_t;
typedef struct _query_dat qdata_t;
typedef struct _query_ctl qctl_t;
    

/* 
 * prototypes 
 */
void query_ondemand(int);
int qrysend(int, uint16_t, uint16_t, char *, size_t);
qreq_t * qryrecv(int); 



#endif /* _COMO_QUERY_H */
