/*
 * Copyright (c) 2004 Intel Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: counter.c,v 1.12 2005/01/23 19:52:21 gianluca Exp $
 */

/*
 * Counter Module
 *
 * This module computes the number of captured packets and bytes
 * Output: packets/bytes seen since last capture interval (1 sec)
 *
 *  [DEPRECATED] Collector formats for this module are:
 *  . "%u %llu %llu" (ts, bytes, packets)
 *
 */

#include <time.h>
#include "como.h"
#include "module.h"

#define REC_SIZE    20      /* size of capture/export record */

#define FLOWDESC    struct _proto_stat

FLOWDESC {
    rec_t       r;	/* XXX shouldn't be known, used here!! */
    uint32_t    ts;
    uint64_t    byts;
    uint64_t    pkts;
};


static int
update(pkt_t *pkt, rec_t *fh, int isnew)
{
    FLOWDESC *x = F(fh);

    if (isnew) {
        x->byts = 0;
        x->pkts = 0;
    }

    x->ts = TS2SEC(pkt->ts);
    x->byts += H16(IP(len));
    x->pkts++;

    return 0;
}

static ssize_t
store(rec_t *rp, char *buf, size_t len)
{
    FLOWDESC *x = F(rp);

    if (len < REC_SIZE) 
	return -1; 

    PUTH32(buf, x->ts);
    PUTH64(buf + 4, x->byts);
    PUTH64(buf + 12, x->pkts);

    return REC_SIZE;
}

static size_t
load(char * buf, size_t len, timestamp_t * ts)
{
    if (len < REC_SIZE) {
        ts = 0;
        return 0;
    }

    *ts = TIME2TS(ntohl(*(uint32_t *)buf), 0);
    return REC_SIZE;
}

static char *
print(char *buf)
{
    FLOWDESC *x = F(buf - sizeof(rec_t));
    static char s[512];
    
    time_t ts = (time_t)ntohl(x->ts);

    sprintf(s, "Ts: %.24s Byte/s: %llu\tPkt/sec:%llu\n",
	asctime(localtime(&ts)), NTOHLL(x->byts), NTOHLL(x->pkts));
    return s;
};

callbacks_t callbacks = {
    sizeof(FLOWDESC),
    0, 
    NULL,
    NULL,
    NULL,
    NULL,
    update,
    NULL,
    NULL,
    NULL,
    NULL,
    store,
    load,
    print
};
